--------------------------------------------------------------------------------
--
-- CIRCUITO TOP
--
--  FERNANDO GEHM MORAES 	    03/agosto/2005
--  CSAR AUGUSTO MISSIO MARCON  10/abril/2008
--  NEY LAERT VILAR CALAZANS     28/abril/2008
--		Corrigida discrepncia entre posio dos BTNS. Agora eles so realmente 
--		lidos para os bits 3 downto 1 do acumulador
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_Logic_Unsigned.all;
use IEEE.Std_Logic_Arith.all;

entity Top is
	port
	(
		ck: in std_logic;
		reset: in std_logic;
		dipsw: in std_logic_vector(7 downto 0);
		btns: in std_logic_vector(3 downto 1);
		leds: out std_logic_vector(7 downto 0);
		an: out std_logic_vector(4 downto 1);
		dec_ddp: out std_logic_vector(7 downto 0)
	);
end Top;

architecture Top of Top is

	signal halt, hold, ce, rw, ceCPU, rwCPU, ceES, rwES: std_logic;
	signal datainCPU, dataoutCPU, dataoutRAM: std_logic_vector(7 downto 0);
	signal address: std_logic_vector(7 downto 0);
	signal d1_CPU, d2_CPU: std_logic_vector(7 downto 0); -- output to displays
	signal d1, d2, d3, d4: std_logic_vector(6 downto 0);
	signal CLK_DIV: std_logic := '0'; -- only for simulation

begin

	process(ck)
	begin
		if ck'event and ck = '1' then
			CLK_DIV <= not CLK_DIV;
		end if;
	end process;

	cpuC: entity work.Cleopatra port map
	(
		clock => CLK_DIV,
		reset => reset,
		halt => halt,
		hold => hold,
		ce => ce,
		rw => rw,
		address => address,
		datain => datainCPU,
		dataout => dataoutCPU
	);
	hold <= '0'; -- Para garantir que processador no  impedido de progredir,

	ramC: entity work.RAM_mem port map
	(
		clock => CLK_DIV,
		ce => ceCPU,
		rw => rwCPU,
		address => address,
		datain => dataoutCPU,
		dataout => dataoutRAM
	);

	deco: entity work.es_decoder port map
	(
		ce => ce,
		rw => rw,
		address => address,
		ceCPU => ceCPU,
		rwCPU => rwCPU,
		ceES => ceES,
		rwES => rwES
	);

-------------------------------------------------------------------------------
-- Comando que captura o dado de algum dispositivo externo  CPU Clepatra.
-- Este comando  disparado por uma instruo LDA executada pela Clepatra.
-- O dado lido pode vir da memria ou de algum dispositivo de entrada, quais
-- sejam as 8 chaves deslizantes ou uma combinao dos 3 botes BTN1 a BTN3.
-- Recomenda-se usar as chaves deslizantes para entrada de dados e os botes
-- para entrada de sinais de controle.
-- Mapa de leitura:
--    Leitura de endereos 00h a 0FBh => leitura de posio da memria principal
--    Leitura de FC => captura valor de BTN3 a BTN1 colocando nos bits
--                     respectivos do Acumulador (3 downto 1)
--    Leitura de FD => captura valor das chaves deslizantes
--    Leitura de FE => sem efeito nesta implementao
--    Leitura de FF => sem efeito nesta implementao
-------------------------------------------------------------------------------
	datainCPU <= dataoutRAM when ceCPU = '1' else
				"0000" & btns & '0' when ceES = '1' and address = x"FC" else
				dipsw when ceES = '1' and address = x"FD" else
				(others => 'Z');

-------------------------------------------------------------------------------
-- Processo que captura o dado escrito pela CPU em dispositivos de sada
--
-- d1_CPU, d2_CPU e leds so trs registradores de 8 bits que recebem dados
-- via instrues STA do processador e disponibilizam sua sadas em
-- dispositivos de E/S (displays e leds).
-- Mapa de escrita:
--    Escrita em endereos 00h a 0FBh => escrita em posio da memria
--    principal
--    Escrita em FC => sem efeito nesta implementao
--    Escrita em FD => efeito nos 8 leds
--    Escrita em FE => efeito nos 2 displays mais  esquerda
--    Escrita em FF => efeito nos 2 displays mais  direita
-------------------------------------------------------------------------------
	process(CLK_DIV, reset)
	begin
		if reset = '1' then
			d1_CPU <= (others => '0');
			d2_CPU <= (others => '0');
			leds <= (others => '0');
		elsif CLK_DIV'event and CLK_DIV = '1' then
			if ceES = '1' and rwES = '0' then
			-- seno, envia para os leds o contedo do acumulador
				if address = x"FD" then
					leds <= dataoutCPU;
				end if;
				if address = x"FE" then
					d1_CPU <= dataoutCPU;
				end if;
				if address = x"FF" then
					d2_CPU <= dataoutCPU;
				end if;
			end if;
		end if;
	end process;

	d4 <= "10" & d1_CPU(7 downto 4) & '1';
	d3 <= "10" & d1_CPU(3 downto 0) & '1';
	d2 <= "10" & d2_CPU(7 downto 4) & '1';
	d1 <= "10" & d2_CPU(3 downto 0) & '1';

	DISPLAY: entity work.dio4_dspl_drv port map
	(
		clock => ck,
		reset => reset,
		d4 => d4,
		d3 => d3,
		d2 => d2,
		d1 => d1,
		an => an,
		dec_ddp => dec_ddp
	);

end Top;
