--------------------------------------------------------------------------------
--
-- Unidade lgico-aritmtica (ULA)
--
-- A ULA possui duas entradas de dados (A_bus e B_bus) e uma saida de dados 
-- (C_bus), um sinal de controle de operao a executar (ulaOp) e quatro sadas 
-- com valores instantneos computados para os 4 qualificadores de estado 
-- (n_inst, z_inst, c_inst, v_inst).
--
--   FERNANDO GEHM MORAES         03/agosto/2005
--   NEY LAERT VILAR CALAZANS     18/junho/2006
--   CSAR AUGUSTO MISSIO MARCON  10/abril/2008
--
--------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use work.PacoteCleopatra.all;

entity Ula is
	port
	(
		A_bus, B_bus: in internal_bus;
		ulaOp: in std_logic_vector(2 downto 0);
		C_Bus: out internal_bus;
		n_inst, z_inst, c_inst, v_inst: out std_logic
	);
end Ula;

-- barramentos internos possuem um bit a mais para permitir clculo dos flags
-- de vai-um e transbordo
architecture Ula of Ula is

	signal A_bus_int, B_bus_int, C_bus_int: std_logic_vector(bus_size downto 0);

begin

	A_bus_int <= '0' & A_bus;
	B_bus_int <= '0' & B_bus;
	C_bus <= C_bus_int(bus_size-1 downto 0);
	process(ulaOp, A_bus_int, B_bus_int)
	begin
		case ulaOp is
			when "000" => C_bus_int <= A_bus_int + B_bus_int;    -- soma
			when "001" => C_bus_int <= A_bus_int + "000000001";  -- incremento
			when "010" => C_bus_int <= not A_bus_int;            -- not
			when "100" => C_bus_int <= B_bus_int;                -- passa B
			when "101" => C_bus_int <= A_bus_int or B_bus_int;   -- ou lgico
			when "110" => C_bus_int <= A_bus_int and B_bus_int;  -- e lgico
			when "111" => C_bus_int <= A_bus_int;                -- passa A
			when others => C_bus_int <= A_bus_int;               -- passa A;
		end case;
	end process;

	-- o bit menos significativo-1  o valor instantneo do qualificador n
	n_inst <= C_bus_int (bus_size-1);

	-- o qualificador z em 1 indica que o resultado da operao  o valor 0
	z_inst <= '1' when (C_bus_int(bus_size-1 downto 0) = "00000000") else '0';

	-- o bit menos significativo  o valor instantneo do qualificador c
	c_inst <= C_bus_int(bus_size);

	-- o qualificador v vai para 1 se os operandos tm sinais iguais
	-- e se este sinal  diferente do sinal do resultado
	v_inst <= '1' when (A_bus_int(bus_size - 1) = B_bus_int(bus_size - 1) and
			A_bus_int(bus_size - 1) /= C_bus_int(bus_size - 1)) else '0';

end Ula;
